#! /usr/bin/env python3
# -*- coding: utf-8 -*-

import sys
import gettext
import requests
from ikabot.helpers.pedirInfo import read
from ikabot.helpers.gui import *
import ikabot.config as config

t = gettext.translation('proxy', config.localedir, languages=config.languages, fallback=True)
_ = t.gettext


def show_proxy(session):
    session_data = session.getSessionData()
    msg = _('using proxy:')
    if 'proxy' in session_data and session_data['proxy']['set'] is True:
        curr_proxy = session_data['proxy']['conf']['https']
        if test_proxy(session_data['proxy']['conf']) is False:
            session_data['proxy']['set'] = False
            session.setSessionData(session_data)
            sys.exit(_('the {} proxy does not work, it has been removed').format(curr_proxy))
        if msg not in config.update_msg:
            # add proxy message
            config.update_msg += '{} {}\n'.format(msg, curr_proxy)
        else:
            # delete old proxy message
            config.update_msg = config.update_msg.replace('\n'.join(config.update_msg.split('\n')[-2:]), '')
            # add new proxy message
            config.update_msg += '{} {}\n'.format(msg, curr_proxy)
    elif msg in config.update_msg:
        # delete old proxy message
        config.update_msg = config.update_msg.replace('\n'.join(config.update_msg.split('\n')[-2:]), '')


def test_proxy(proxy_dict):
    try:
        requests.get('https://lobby.ikariam.gameforge.com/', proxies=proxy_dict, verify=config.do_ssl_verify)
    except Exception:
        return False
    return True


def read_proxy():
    print(_('Enter the proxy (examples: socks5://127.0.0.1:9050, https://45.117.163.22:8080):'))
    proxy_str = read(msg='proxy: ')
    proxy_dict = {'http': proxy_str, 'https': proxy_str}
    if test_proxy(proxy_dict) is False:
        print(_('The proxy does not work.'))
        enter()
        return None
    print(_('The proxy works and it will be used for all future requests.'))
    enter()
    return proxy_dict


def proxyConf(session, event, stdin_fd, predetermined_input):
    """
    Parameters
    ----------
    session : ikabot.web.session.Session
    event : multiprocessing.Event
    stdin_fd: int
    predetermined_input : multiprocessing.managers.SyncManager.list
    """
    sys.stdin = os.fdopen(stdin_fd)
    config.predetermined_input = predetermined_input
    try:
        banner()
        print(_('Warning: The proxy does not apply to the requests sent to the lobby!\n'))

        session_data = session.getSessionData()
        if 'proxy' not in session_data or session_data['proxy']['set'] is False:
            print(_('Right now, there is no proxy configured.'))
            proxy_dict = read_proxy()
            if proxy_dict is None:
                event.set()
                return
            session_data['proxy'] = {}
            session_data['proxy']['conf'] = proxy_dict
            session_data['proxy']['set'] = True
        else:
            curr_proxy = session_data['proxy']['conf']['https']
            print(_('Current proxy: {}').format(curr_proxy))
            print(_('What do you want to do?'))
            print(_('0) Exit'))
            print(_('1) Set a new proxy'))
            print(_('2) Remove the current proxy'))
            rta = read(min=0, max=2)

            if rta == 0:
                event.set()
                return
            if rta == 1:
                proxy_dict = read_proxy()
                if proxy_dict is None:
                    event.set()
                    return
                session_data['proxy']['conf'] = proxy_dict
                session_data['proxy']['set'] = True
            if rta == 2:
                session_data['proxy']['set'] = False
                print(_('The proxy has been removed.'))
                enter()

        session.setSessionData(session_data)
        event.set()
    except KeyboardInterrupt:
        event.set()
        return



#! /usr/bin/env python3
# -*- coding: utf-8 -*-

import re
import math
import json
import gettext
import traceback
from decimal import *
from ikabot.helpers.process import set_child_mode
from ikabot.helpers.varios import addThousandSeparator
from ikabot.helpers.gui import enter, banner
from ikabot.helpers.getJson import getCity
from ikabot.helpers.signals import setInfoSignal
from ikabot.helpers.planRoutes import waitForArrival
from ikabot.helpers.pedirInfo import getIdsOfCities, read
from ikabot.config import *
from ikabot.helpers.botComm import *
from ikabot.helpers.resources import *
from ikabot.helpers.market import *


t = gettext.translation('buyResources', localedir, languages=languages, fallback=True)
_ = t.gettext


def chooseResource(session, city):
    """
    Parameters
    ----------
    session : ikabot.web.session.Session
    city : dict
    """
    print(_('Which resource do you want to buy?'))
    for index, material_name in enumerate(materials_names):
        print('({:d}) {}'.format(index+1, material_name))
    choise = read(min=1, max=5)
    resource = choise - 1
    if resource == 0:
        resource = 'resource'
    data = {
        'cityId': city['id'],
        'position': city['pos'],
        'view': 'branchOffice',
        'activeTab': 'bargain',
        'type': 444,
        'searchResource': resource,
        'range': city['rango'],
        'backgroundView': 'city',
        'currentCityId': city['id'],
        'templateView': 'branchOffice',
        'currentTab': 'bargain',
        'actionRequest': actionRequest,
        'ajax': 1
    }
    # this will set the chosen resource in the store
    session.post(payloadPost=data)
    resource = choise - 1
    # return the chosen resource
    return resource


def getOffers(session, city):
    """
    Parameters
    ----------
    session : ikabot.web.session.Session
    city : dict
    Returns
    -------
    offers : list[dict]
    """
    html = getMarketHtml(session, city)
    hits = re.findall(r'short_text80">(.*?) *<br/>\((.*?)\)\s *</td>\s *<td>(\d+)</td>\s *<td>(.*?)/td>\s *<td><img src="skin/resources/icon_(\w+)\.png[\s\S]*?white-space:nowrap;">(\d+)\s[\s\S]*?href="\?view=takeOffer&destinationCityId=(\d+)&oldView=branchOffice&activeTab=bargain&cityId=(\d+)&position=(\d+)&type=(\d+)&resource=(\w+)"', html)
    offers = []
    for hit in hits:
        offer = {
            'ciudadDestino': hit[0],
            'jugadorAComprar': hit[1],
            'bienesXminuto': int(hit[2]),
            'amountAvailable': int(hit[3].replace(',', '').replace('.', '').replace('<', '')),
            'tipo': hit[4],
            'precio': int(hit[5]),
            'destinationCityId': hit[6],
            'cityId': hit[7],
            'position': hit[8],
            'type': hit[9],
            'resource': hit[10]
        }
        offers.append(offer)
    return offers


def calculateCost(offers, total_amount_to_buy):
    """
    Parameters
    ----------
    offers : list[dict]
    total_amount_to_buy : int
    Returns
    -------
    total_cost : int
    """
    total_cost = 0
    for offer in offers:
        if total_amount_to_buy == 0:
            break
        buy_amount = min(offer['amountAvailable'], total_amount_to_buy)
        total_amount_to_buy -= buy_amount
        total_cost += buy_amount * offer['precio']
    return total_cost


def chooseCommertialCity(commercial_cities):
    """
    Parameters
    ----------
    commercial_cities : list[dict]

    Returns
    -------
    commercial_city : dict
    """
    print(_('From which city do you want to buy resources?\n'))
    for i, city in enumerate(commercial_cities):
        print('({:d}) {}'.format(i + 1, city['name']))
    selected_city_index = read(min=1, max=len(commercial_cities))
    return commercial_cities[selected_city_index - 1]


def buyResources(session, event, stdin_fd, predetermined_input):
    """
    Parameters
    ----------
    session : ikabot.web.session.Session
    event : multiprocessing.Event
    stdin_fd: int
    predetermined_input : multiprocessing.managers.SyncManager.list
    """
    sys.stdin = os.fdopen(stdin_fd)
    config.predetermined_input = predetermined_input
    try:
        banner()

        # get all the cities with a store
        commercial_cities = getCommercialCities(session)
        if len(commercial_cities) == 0:
            print(_('There is no store build'))
            enter()
            event.set()
            return

        # choose which city to buy from
        if len(commercial_cities) == 1:
            city = commercial_cities[0]
        else:
            city = chooseCommertialCity(commercial_cities)
            banner()

        # choose resource to buy
        resource = chooseResource(session, city)
        banner()

        # get all the offers of the chosen resource from the chosen city
        offers = getOffers(session, city)
        if len(offers) == 0:
            print(_('There are no offers available.'))
            enter()
            event.set()
            return

        # display offers to the user
        total_price = 0
        total_amount = 0
        for offer in offers:
            amount = offer['amountAvailable']
            price = offer['precio']
            cost = amount * price
            print(_('amount:{}').format(addThousandSeparator(amount)))
            print(_('price :{:d}').format(price))
            print(_('cost  :{}').format(addThousandSeparator(cost)))
            print('')
            total_price += cost
            total_amount += amount

        # ask how much to buy
        print(_('Total amount available to purchase: {}, for {}').format(addThousandSeparator(total_amount), addThousandSeparator(total_price)))
        available = city['freeSpaceForResources'][resource]
        if available < total_amount:
            print(_('You just can buy {} due to storing capacity').format(addThousandSeparator(available)))
            total_amount = available
        print('')
        amount_to_buy = read(msg=_('How much do you want to buy?: '), min=0, max=total_amount)
        if amount_to_buy == 0:
            event.set()
            return

        # calculate the total cost
        (gold, __) = getGold(session, city)
        total_cost = calculateCost(offers, amount_to_buy)

        print(_('\nCurrent gold: {}.\nTotal cost  : {}.\nFinal gold  : {}.'). format(addThousandSeparator(gold), addThousandSeparator(total_cost), addThousandSeparator(gold - total_cost)))
        print(_('Proceed? [Y/n]'))
        rta = read(values=['y', 'Y', 'n', 'N', ''])
        if rta.lower() == 'n':
            event.set()
            return

        print(_('It will be purchased {}').format(addThousandSeparator(amount_to_buy)))
        enter()
    except KeyboardInterrupt:
        event.set()
        return

    set_child_mode(session)
    event.set()

    info = _('\nI will buy {} from {} to {}\n').format(addThousandSeparator(amount_to_buy), materials_names[resource], city['cityName'])
    setInfoSignal(session, info)
    try:
        do_it(session, city, offers, amount_to_buy)
    except Exception as e:
        msg = _('Error in:\n{}\nCause:\n{}').format(info, traceback.format_exc())
        sendToBot(session, msg)
    finally:
        session.logout()


def buy(session, city, offer, amount_to_buy):
    """
    Parameters
    ----------
    session : ikabot.web.session.Session
    city : dict
    offer : dict
    amount_to_buy : int
    """
    ships = int(math.ceil((Decimal(amount_to_buy) / Decimal(500))))
    data_dict = {
        'action': 'transportOperations',
        'function': 'buyGoodsAtAnotherBranchOffice',
        'cityId': offer['cityId'],
        'destinationCityId': offer['destinationCityId'],
        'oldView': 'branchOffice',
        'position': city['pos'],
        'avatar2Name': offer['jugadorAComprar'],
        'city2Name': offer['ciudadDestino'],
        'type': int(offer['type']),
        'activeTab': 'bargain',
        'transportDisplayPrice': 0,
        'premiumTransporter': 0,
        'capacity': 5,
        'max_capacity': 5,
        'jetPropulsion': 0,
        'transporters': ships,
        'backgroundView': 'city',
        'currentCityId': offer['cityId'],
        'templateView': 'takeOffer',
        'currentTab': 'bargain',
        'actionRequest': actionRequest,
        'ajax': 1
    }
    url = 'view=takeOffer&destinationCityId={}&oldView=branchOffice&activeTab=bargain&cityId={}&position={}&type={}&resource={}&backgroundView=city&currentCityId={}&templateView=branchOffice&actionRequest={}&ajax=1'.format(offer['destinationCityId'], offer['cityId'], offer['position'], offer['type'], offer['resource'], offer['cityId'], actionRequest)
    data = session.post(url)
    html = json.loads(data, strict=False)[1][1][1]
    hits = re.findall(r'"tradegood(\d)Price"\s*value="(\d+)', html)
    for hit in hits:
        data_dict['tradegood{}Price'.format(hit[0])] = int(hit[1])
        data_dict['cargo_tradegood{}'.format(hit[0])] = 0
    hit = re.search(r'"resourcePrice"\s*value="(\d+)', html)
    if hit:
        data_dict['resourcePrice'] = int(hit.group(1))
        data_dict['cargo_resource'] = 0
    resource = offer['resource']
    if resource == 'resource':
        data_dict['cargo_resource'] = amount_to_buy
    else:
        data_dict['cargo_tradegood{}'.format(resource)] = amount_to_buy
    session.post(payloadPost=data_dict)
    msg = _('I buy {} to {} from {}').format(addThousandSeparator(amount_to_buy), offer['ciudadDestino'], offer['jugadorAComprar'])
    sendToBotDebug(session, msg, debugON_buyResources)


def do_it(session, city, offers, amount_to_buy):
    """
    Parameters
    ----------
    session : ikabot.web.session.Session
    city : dict
    offers : list[dict]
    amount_to_buy : int
    """
    while True:
        for offer in offers:
            if amount_to_buy == 0:
                return
            if offer['amountAvailable'] == 0:
                continue

            ships_available = waitForArrival(session)
            storageCapacity = ships_available * 500
            buy_amount = min(amount_to_buy, storageCapacity, offer['amountAvailable'])

            amount_to_buy -= buy_amount
            offer['amountAvailable'] -= buy_amount
            buy(session, city, offer, buy_amount)
            # start from the beginning again, so that we always buy from the cheapest offers fisrt
            break



#! /usr/bin/env python3
# -*- coding: utf-8 -*-

import gettext
import sys
from ikabot.config import *
from ikabot.helpers.gui import *
from ikabot.helpers.pedirInfo import read
from ikabot.helpers.getJson import getCity

t = gettext.translation('vacationMode', localedir, languages=languages, fallback=True)
_ = t.gettext


def activateVacationMode(session):
    """
    Parameters
    ----------
    session : ikabot.web.session.Session
    """
    html = session.get()
    city = getCity(html)

    data = {'action': 'Options', 'function': 'activateVacationMode', 'actionRequest': actionRequest, 'backgroundView': 'city', 'currentCityId': city['id'], 'templateView': 'options_umod_confirm'}
    session.post(params=data, ignoreExpire=True)


def vacationMode(session, event, stdin_fd, predetermined_input):
    """
    Parameters
    ----------
    session : ikabot.web.session.Session
    event : multiprocessing.Event
    stdin_fd: int
    predetermined_input : multiprocessing.managers.SyncManager.list
    """
    sys.stdin = os.fdopen(stdin_fd)
    config.predetermined_input = predetermined_input
    try:
        banner()
        print(_('Activate vacation mode? [Y/n]'))
        rta = read(values=['y', 'Y', 'n', 'N', ''])
        if rta.lower() == 'n':
            event.set()
            return

        activateVacationMode(session)

        print(_('Vacation mode has been activated.'))
        enter()
        event.set()
        clear()
        sys.exit()
    except KeyboardInterrupt:
        event.set()
        return



#! /usr/bin/env python3
# -*- coding: utf-8 -*-

import json
import gettext
import traceback
from ikabot.config import *
from ikabot.helpers.gui import *
from ikabot.helpers.varios import *
from ikabot.helpers.botComm import *
from ikabot.helpers.pedirInfo import *
from ikabot.helpers.process import set_child_mode
from ikabot.helpers.getJson import getCity
from ikabot.helpers.signals import setInfoSignal

t = gettext.translation('investigate', localedir, languages=languages, fallback=True)
_ = t.gettext


def get_studies(session):
    html = session.get()
    city = getCity(html)
    city_id = city['id']
    url = 'view=researchAdvisor&oldView=updateGlobalData&cityId={0}&backgroundView=city&currentCityId={0}&templateView=researchAdvisor&actionRequest={1}&ajax=1'.format(city_id, actionRequest)
    resp = session.post(url)
    resp = json.loads(resp, strict=False)
    return resp[2][1]


def study(session, studies, num_study):
    html = session.get()
    city = getCity(html)
    city_id = city['id']
    research_type = studies['js_researchAdvisorChangeResearchType{}'.format(num_study)]['ajaxrequest'].split('=')[-1]
    url = 'action=Advisor&function=doResearch&actionRequest={}&type={}&backgroundView=city&currentCityId={}&templateView=researchAdvisor&ajax=1'.format(actionRequest, research_type, city_id)
    session.post(url)


def investigate(session, event, stdin_fd, predetermined_input):
    """
    Parameters
    ----------
    session : ikabot.web.session.Session
    event : multiprocessing.Event
    stdin_fd: int
    predetermined_input : multiprocessing.managers.SyncManager.list
    """
    sys.stdin = os.fdopen(stdin_fd)
    config.predetermined_input = predetermined_input
    try:
        banner()

        studies = get_studies(session)
        keys = list(studies.keys())
        num_studies = len([key for key in keys if 'js_researchAdvisorChangeResearchTypeTxt' in key])

        available = []
        for num_study in range(num_studies):
            if 'js_researchAdvisorProgressTxt{}'.format(num_study) in studies:
                available.append(num_study)

        if len(available) == 0:
            print(_('There are no available studies.'))
            enter()
            event.set()
            return

        print(_('Which one do you wish to study?'))
        print('0) None')
        for index, num_study in enumerate(available):
            print('{:d}) {}'.format(index+1, studies['js_researchAdvisorNextResearchName{}'.format(num_study)]))
        choice = read(min=0, max=len(available))

        if choice == 0:
            event.set()
            return

        study(session, studies, available[choice-1])
        print(_('Done.'))
        enter()
        event.set()

    except KeyboardInterrupt:
        event.set()
        return



#! /usr/bin/env python3
# -*- coding: utf-8 -*-

import re
import json
from decimal import *
from ikabot.config import *

getcontext().prec = 30


def getAvailableResources(html, num=False):
    """
    Parameters
    ----------
    html : string

    Returns
    -------
    resources_available : list[int] | list[str]
    """
    resources = re.search(r'\\"resource\\":(\d+),\\"2\\":(\d+),\\"1\\":(\d+),\\"4\\":(\d+),\\"3\\":(\d+)}', html)
    if num:
        return [int(resources.group(1)), int(resources.group(3)), int(resources.group(2)), int(resources.group(5)), int(resources.group(4))]
    else:
        return [resources.group(1), resources.group(3), resources.group(2), resources.group(5), resources.group(4)]


def getWarehouseCapacity(html):
    """
    Parameters
    ----------
    html : string
    Returns
    -------
    capacity : int
    """
    capacity = re.search(r'maxResources:\s*JSON\.parse\(\'{\\"resource\\":(\d+),', html).group(1)
    return int(capacity)


def getWineConsumption(html):
    """
    Parameters
    ----------
    html : string
    Returns
    -------
    capacity : int
    """
    result = re.search(r'GlobalMenu_WineConsumption"\s*class="rightText">\s*(\d+)\s', html)
    if result:
        return int(result.group(1))
    return 0


def getProductionPerSecond(session, city_id):
    """
    Parameters
    ----------
    session : ikabot.web.session.Session
    city_id : int

    Returns
    -------
    production: tuple[Decimal, Decimal, int]
    """
    prod = session.post(payloadPost={'action': 'header', 'function': 'changeCurrentCity', 'actionRequest': actionRequest, 'cityId': city_id, 'ajax': '1'})
    prod = json.loads(prod, strict=False)
    prod = prod[0][1]['headerData']
    wood_production = Decimal(prod['resourceProduction'])
    luxury_production = Decimal(prod['tradegoodProduction'])
    luxury_resource_type = int(prod['producedTradegood'])

    return wood_production, luxury_production, luxury_resource_type



#! /usr/bin/env python3
# -*- coding: utf-8 -*-

import re
import math
import json
import gettext
import sys
from ikabot.helpers.pedirInfo import read
from ikabot import config
from decimal import *
from ikabot.config import *
from ikabot.helpers.gui import *
from ikabot.helpers.naval import *
from ikabot.helpers.varios import *

t = gettext.translation('shipMovements', localedir, languages=languages, fallback=True)
_ = t.gettext


def isHostile(movement):
    """
    Parameters
    ----------
    movement : dict

    Returns
    -------
    is hostile : bool
    """
    if movement['army']['amount']:
        return True
    for mov in movement['fleet']['ships']:
        if mov['cssClass'] != 'ship_transport':
            return True
    return False


def shipMovements(session, event, stdin_fd, predetermined_input):
    """
    Parameters
    ----------
    session : ikabot.web.session.Session
    event : multiprocessing.Event
    stdin_fd: int
    predetermined_input : multiprocessing.managers.SyncManager.list
    """
    sys.stdin = os.fdopen(stdin_fd)
    config.predetermined_input = predetermined_input
    try:
        banner()

        print(_('Ships {:d}/{:d}\n').format(getAvailableShips(session), getTotalShips(session)))

        cityId = getCurrentCityId(session)
        url = 'view=militaryAdvisor&oldView=city&oldBackgroundView=city&backgroundView=city&currentCityId={}&actionRequest={}&ajax=1'.format(cityId, actionRequest)
        resp = session.post(url)
        resp = json.loads(resp, strict=False)
        movements = resp[1][1][2]['viewScriptParams']['militaryAndFleetMovements']
        time_now = int(resp[0][1]['time'])

        if len(movements) == 0:
            print(_('There are no movements'))
            enter()
            event.set()
            return

        for movement in movements:

            color = ''
            if movement['isHostile']:
                color = bcolors.RED + bcolors.BOLD
            elif movement['isOwnArmyOrFleet']:
                color = bcolors.BLUE + bcolors.BOLD
            elif movement['isSameAlliance']:
                color = bcolors.GREEN + bcolors.BOLD

            origin = '{} ({})'.format(movement['origin']['name'], movement['origin']['avatarName'])
            destination = '{} ({})'.format(movement['target']['name'], movement['target']['avatarName'])
            arrow = '<-' if movement['event']['isFleetReturning'] else '->'
            time_left = int(movement['eventTime']) - time_now
            print('{}{} {} {}: {} ({}) {}'.format(color, origin, arrow, destination, movement['event']['missionText'], daysHoursMinutes(time_left), bcolors.ENDC))

            if movement['isHostile']:
                troops = movement['army']['amount']
                fleets = movement['fleet']['amount']
                print(_('Troops:{}\nFleets:{}').format(addThousandSeparator(troops), addThousandSeparator(fleets)))
            elif isHostile(movement):
                troops = movement['army']['amount']
                ships = 0
                fleets = 0
                for mov in movement['fleet']['ships']:
                    if mov['cssClass'] == 'ship_transport':
                        ships += int(mov['amount'])
                    else:
                        fleets += int(mov['amount'])
                print(_('Troops:{}\nFleets:{}\n Ships:{}').format(addThousandSeparator(troops), addThousandSeparator(fleets), addThousandSeparator(ships)))
            else:
                assert len(materials_names) == 5
                total_load = 0
                for resource in movement['resources']:
                    amount = resource['amount']
                    tradegood = resource['cssClass'].split()[1]
                    # gold won't be translated
                    if tradegood != 'gold':
                        index = materials_names_tec.index(tradegood)
                        tradegood = materials_names[index]
                    total_load += int(amount.replace(',', '').replace('.', ''))
                    print(_('{} of {}').format(amount, tradegood))
                ships = int(math.ceil((Decimal(total_load) / Decimal(500))))
                print(_('{:d} Ships').format(ships))
        enter()
        event.set()
    except KeyboardInterrupt:
        event.set()
        return



#! /usr/bin/env python3
# -*- coding: utf-8 -*-

import re
import gettext
import json
from ikabot.config import *
from ikabot.helpers.pedirInfo import *
from ikabot.helpers.getJson import *
from ikabot.helpers.resources import *
from ikabot.helpers.gui import *
from ikabot.helpers.varios import *

t = gettext.translation('donate', localedir, languages=languages, fallback=True)
_ = t.gettext


def donate(session, event, stdin_fd, predetermined_input):
    """
    Parameters
    ----------
    session : ikabot.web.session.Session
    event : multiprocessing.Event
    stdin_fd: int
    predetermined_input : multiprocessing.managers.SyncManager.list
    """
    sys.stdin = os.fdopen(stdin_fd)
    config.predetermined_input = predetermined_input
    try:
        banner()

        city = chooseCity(session)
        banner()

        woodAvailable = city['recursos'][0]

        islandId = city['islandId']
        html = session.get(island_url + islandId)
        island = getIsland(html)

        island_type = island['tipo']
        resource_name = tradegoods_names[0]
        tradegood_name = tradegoods_names[int(island_type)]

        # get resource information
        url = 'view=resource&type=resource&islandId={0}&backgroundView=island&currentIslandId={0}&actionRequest={1}&ajax=1'.format(islandId, actionRequest)
        resp = session.post(url)
        resp = json.loads(resp, strict=False)

        resourceLevel = resp[0][1]['backgroundData']['resourceLevel']
        tradegoodLevel = resp[0][1]['backgroundData']['tradegoodLevel']
        resourceEndUpgradeTime = int(resp[0][1]['backgroundData']['resourceEndUpgradeTime'])
        resourceUpgrading = resourceEndUpgradeTime > 0
        tradegoodEndUpgradeTime = int(resp[0][1]['backgroundData']['tradegoodEndUpgradeTime'])
        tradegoodUpgrading = tradegoodEndUpgradeTime > 0

        if resourceUpgrading:
            resourceUpgradeMsg = _('(upgrading, ends in:{})').format(daysHoursMinutes(resourceEndUpgradeTime))
        else:
            resourceUpgradeMsg = ''
        if tradegoodUpgrading:
            tradegoodUpgradeMsg = _('(upgrading, ends in:{})').format(daysHoursMinutes(tradegoodEndUpgradeTime))
        else:
            tradegoodUpgradeMsg = ''

        html = resp[1][1][1]
        wood_total_needed, wood_donated = re.findall(r'<li class="wood">(.*?)</li>', html)
        wood_total_needed = wood_total_needed.replace(',', '').replace('.', '')
        wood_total_needed = int(wood_total_needed)
        wood_donated = wood_donated.replace(',', '').replace('.', '')
        wood_donated = int(wood_donated)

        if resourceUpgrading and tradegoodUpgrading:
            print(_('Both the {} (ends in:{}) and the {} (ends in:{}) are being upgraded rigth now.\n'.format(resource_name, daysHoursMinutes(resourceEndUpgradeTime), tradegood_name, daysHoursMinutes(tradegoodEndUpgradeTime))))
            enter()
            event.set()
            return

        print('{} lv:{} {}'.format(resource_name, resourceLevel, resourceUpgradeMsg))
        print('{} / {} ({}%)\n'.format(addThousandSeparator(wood_donated), addThousandSeparator(wood_total_needed), addThousandSeparator(int((100 * wood_donated) / wood_total_needed))))

        # get tradegood information
        url = 'view=tradegood&type={0}&islandId={1}&backgroundView=island&currentIslandId={1}&actionRequest={2}&ajax=1'.format(island_type, islandId, actionRequest)
        resp = session.post(url)

        resp = json.loads(resp, strict=False)
        html = resp[1][1][1]
        tradegood_total_needed, tradegood_donated = re.findall(r'<li class="wood">(.*?)</li>', html)
        tradegood_total_needed = tradegood_total_needed.replace(',', '').replace('.', '')
        tradegood_total_needed = int(tradegood_total_needed)
        tradegood_donated = tradegood_donated.replace(',', '').replace('.', '')
        tradegood_donated = int(tradegood_donated)

        print('{} lv:{} {}'.format(tradegood_name, tradegoodLevel, tradegoodUpgradeMsg))
        print('{} / {} ({}%)\n'.format(addThousandSeparator(tradegood_donated), addThousandSeparator(tradegood_total_needed), addThousandSeparator(int((100 * tradegood_donated) / tradegood_total_needed))))

        print(_('Wood available:{}\n').format(addThousandSeparator(woodAvailable)))

        if resourceUpgrading is False and tradegoodUpgrading is False:
            msg = _('Donate to {} (1) or {} (2)?:').format(resource_name, tradegood_name)
            donation_type = read(msg=msg, min=1, max=2)
            name = resource_name if donation_type == 1 else tradegood_name
            print('')
        else:
            if resourceUpgrading is False and tradegoodUpgrading is True:
                donation_type = 1
                name = resource_name
            else:
                donation_type = 2
                name = tradegood_name
            print('Donate to:{}\n'.format(name))

        donation_type = 'resource' if donation_type == 1 else 'tradegood'

        amount = read(min=0, max=woodAvailable, default=0, additionalValues=['all','half'], msg=_('Amount (number, all, half):'))
        if amount == 'all':
            amount = woodAvailable
        elif amount == 'half':
            amount = woodAvailable // 2
        elif amount == 0:
            event.set()
            return
        print(_('Will donate {} to the {}?').format(addThousandSeparator(amount), name))
        print(_('\nProceed? [Y/n]'))
        rta = read(values=['y', 'Y', 'n', 'N', ''])
        if rta.lower() == 'n':
            event.set()
            return

        # do the donation
        session.post(payloadPost={'islandId': islandId, 'type': donation_type, 'action': 'IslandScreen', 'function': 'donate', 'donation': amount, 'backgroundView': 'island', 'templateView': 'resource', 'actionRequest': actionRequest, 'ajax': '1'})

        print('\nDonation successful.')
        enter()
        event.set()
        return
    except KeyboardInterrupt:
        event.set()
        return



#! /usr/bin/env python3
# -*- coding: utf-8 -*-

import gettext
import sys
from ikabot.helpers.pedirInfo import read
from ikabot.helpers.process import run
from ikabot.helpers.gui import *
from ikabot.config import *

t = gettext.translation('update', localedir, languages=languages, fallback=True)
_ = t.gettext


def update(session, event, stdin_fd, predetermined_input):
    """
    Parameters
    ----------
    session : ikabot.web.session.Session
    event : multiprocessing.Event
    stdin_fd: int
    predetermined_input : multiprocessing.managers.SyncManager.list
    """
    sys.stdin = os.fdopen(stdin_fd)
    config.predetermined_input = predetermined_input
    try:
        print(_('To update ikabot run:'))
        print('python3 -m pip install --user --upgrade ikabot')
        enter()
        event.set()
    except KeyboardInterrupt:
        event.set()
        return



#! /usr/bin/env python3
# -*- coding: utf-8 -*-

import time
import gettext
import traceback
from ikabot.config import *
from ikabot.helpers.botComm import *
from ikabot.helpers.pedirInfo import *
from ikabot.helpers.gui import *
from ikabot.helpers.varios import wait
from ikabot.helpers.process import set_child_mode
from ikabot.helpers.signals import setInfoSignal
from ikabot.helpers.getJson import getCity
from ikabot.helpers.resources import getAvailableResources

t = gettext.translation('donationBot', localedir, languages=languages, fallback=True)
_ = t.gettext


def donationBot(session, event, stdin_fd, predetermined_input):
    """
    Parameters
    ----------
    session : ikabot.web.session.Session
    event : multiprocessing.Event
    stdin_fd: int
    predetermined_input : multiprocessing.managers.SyncManager.list
    """
    sys.stdin = os.fdopen(stdin_fd)
    config.predetermined_input = predetermined_input
    try:
        banner()
        (cities_ids, cities) = getIdsOfCities(session)
        cities_dict = {}
        initials = [material_name[0] for material_name in materials_names]
        print('Enter how often you want to donate in minutes. (min = 1, default = 1 day)')
        waiting_time = read(min=1, digit=True, default= 1 * 24 * 60)
        print('Enter a maximum additional random waiting time between donations in minutes. (min = 0, default = 1 hour)')
        max_random_waiting_time = read(min=0, digit=True, default = 1 * 60)
        for cityId in cities_ids:
            tradegood = cities[cityId]['tradegood']
            initial = initials[int(tradegood)]
            print(_('In {} ({}), Do you wish to donate to the forest, to the trading good or neither? [f/t/n]').format(cities[cityId]['name'], initial))
            f = _('f')
            t = _('t')
            n = _('n')

            rta = read(values=[f, f.upper(), t, t.upper(), n, n.upper()])
            if rta.lower() == f:
                donation_type = 'resource'
            elif rta.lower() == t:
                donation_type = 'tradegood'
            else:
                donation_type = None
                percentage = None

            if donation_type is not None:
                print(_('What is the maximum percentage of your storage capacity that you whish to keep occupied? (the resources that exceed it, will be donated) (default: 80%)'))
                percentage = read(min=0, max=100, empty=True)
                if percentage == '':
                    percentage = 80
                elif percentage == 100:  # if the user is ok with the storage beeing totally full, don't donate at all
                    donation_type = None

            cities_dict[cityId] = {'donation_type': donation_type, 'percentage': percentage}

        print(_('I will donate every {} minutes.'.format(waiting_time)))
        enter()
    except KeyboardInterrupt:
        event.set()
        return

    set_child_mode(session)
    event.set()

    info = _('\nI donate every {} minutes\n'.format(waiting_time))
    setInfoSignal(session, info)
    try:
        do_it(session, cities_ids, cities_dict, waiting_time, max_random_waiting_time)
    except Exception as e:
        msg = _('Error in:\n{}\nCause:\n{}').format(info, traceback.format_exc())
        sendToBot(session, msg)
    finally:
        session.logout()


def do_it(session, cities_ids, cities_dict, waiting_time, max_random_waiting_time):
    """
    Parameters
    ----------
    session : ikabot.web.session.Session
    cities_ids : list[int]
    cities_dict : dict[int, dict]
    waiting_time: int
    max_random_waiting_time: int
    """
    for cityId in cities_ids:
        html = session.get(city_url + cityId)
        city = getCity(html)
        cities_dict[cityId]['island'] = city['islandId']

    while True:
        for cityId in cities_ids:
            donation_type = cities_dict[cityId]['donation_type']
            if donation_type is None:
                continue

            # get the storageCapacity and the wood this city has
            html = session.get(city_url + cityId)
            city = getCity(html)
            wood = city['recursos'][0]
            storageCapacity = city['storageCapacity']

            # get the percentage
            percentage = cities_dict[cityId]['percentage']
            percentage /= 100

            # calculate what is the amount of wood that should be preserved
            max_wood = storageCapacity * percentage
            max_wood = int(max_wood)

            # calculate the wood that is exceeding the percentage
            to_donate = wood - max_wood
            if to_donate <= 0:
                continue

            islandId = cities_dict[cityId]['island']

            # donate
            session.post(payloadPost={'islandId': islandId, 'type': donation_type, 'action': 'IslandScreen', 'function': 'donate', 'donation': to_donate, 'backgroundView': 'island', 'templateView': donation_type, 'actionRequest': actionRequest, 'ajax': '1'})

        msg = _('I donated automatically.')
        sendToBotDebug(session, msg, debugON_donationBot)

        # sleep a day
        wait(waiting_time*60, maxrandom=max_random_waiting_time*60)



#! /usr/bin/env python3
# -*- coding: utf-8 -*-

import re
import sys
import json
import gettext
import os
from ikabot import config
from decimal import *
from ikabot.config import *
from ikabot.helpers.getJson import *
from ikabot.helpers.gui import *

t = gettext.translation('pedirInfo', localedir, languages=languages, fallback=True)
_ = t.gettext

getcontext().prec = 30


def read(min=None, max=None, digit=False, msg=prompt, values=None, empty=False, additionalValues=None, default=None):  # user input
    """Reads input from user
    Parameters
    ----------
    min : int
        smallest number acceptable as input
    max : int
        greatest number acceptable as input
    digit : bool
        boolean indicating whether or not the input MUST be an int
    msg : str
        string printed before the user is asked for input
    values : list
        list of strings which are acceptable as input
    empty : bool
        a boolean indicating whether or not an empty string is acceptable as input
    additionalValues : list
        list of strings which are additional valid inputs. Can be used with digit = True to validate a string as an input among all digits

    Returns
    -------
    result : int | str
        int representing the user's choice
    """
    try:
        if len(config.predetermined_input) != 0:
            return config.predetermined_input.pop(0)
    except Exception:
        pass

    def _invalid():
        print('\033[1A\033[K', end="")  # remove line
        return read(min, max, digit, msg, values, additionalValues=additionalValues)

    try:
        read_input = input(msg)
    except EOFError:
        return _invalid()

    if additionalValues is not None and read_input in additionalValues:
        return read_input

    if read_input == '' and default is not None:
        return default

    if read_input == '' and empty is True:
        return read_input

    if digit is True or min is not None or max is not None:
        if read_input.isdigit() is False:
            return _invalid()
        else:
            try:
                read_input = eval(read_input)
            except SyntaxError:
                return _invalid()
    if min is not None and read_input < min:
        return _invalid()
    if max is not None and read_input > max:
        return _invalid()
    if values is not None and read_input not in values:
        return _invalid()
    return read_input


def chooseCity(session, foreign=False):
    """Prompts the user to chose a city
    Parameters
    ----------
    session : ikabot.web.session.Session
        Session object
    foreign : bool
        lets the user choose a foreign city

    Returns
    -------
    city : City
        a city object representing the chosen city
    """
    global menu_cities
    (ids, cities) = getIdsOfCities(session)
    if menu_cities == '':
        longest_city_name_length = 0
        for city_id in ids:
            length = len(cities[city_id]['name'])
            if length > longest_city_name_length:
                longest_city_name_length = length

        def pad(city_name):
            return ' ' * (longest_city_name_length - len(city_name) + 2)

        resources_abbreviations = {'1': _('(W)'), '2': _('(M)'), '3': _('(C)'), '4': _('(S)')}

        i = 0
        if foreign:
            print(_(' 0: foreign city'))
        else:
            print('')
        for city_id in ids:
            i += 1
            resource_index = cities[city_id]['tradegood']
            resource_abb = resources_abbreviations[resource_index]
            city_name = cities[city_id]['name']
            matches = re.findall(r'u[0-9a-f]{4}', city_name)
            for match in matches:
                to_unicode = '\\' + match
                to_unicode = to_unicode.encode().decode('unicode-escape')
                city_name = city_name.replace(match, to_unicode)
            num = ' ' + str(i) if i < 10 else str(i)
            menu_cities += '{}: {}{}{}\n'.format(num, city_name, pad(city_name), resource_abb)
        menu_cities = menu_cities[:-1]
    if foreign:
        print(_(' 0: foreign city'))
    print(menu_cities)

    if foreign:
        selected_city_index = read(min=0, max=len(ids))
    else:
        selected_city_index = read(min=1, max=len(ids))
    if selected_city_index == 0:
        return chooseForeignCity(session)
    else:
        html = session.get(city_url + ids[selected_city_index - 1])
        return getCity(html)


def chooseForeignCity(session):
    """Prompts the user to select an island, and a city on that island (is only used in chooseCity)
    Parameters
    ----------
    session : ikabot.web.session.Session
        Session object

    Returns
    -------
    city : City
        a city object representing the city the user chose
    """
    banner()
    x = read(msg='coordinate x:', digit=True)
    y = read(msg='coordinate y:', digit=True)
    print('')
    url = 'view=worldmap_iso&islandX={}&islandY={}&oldBackgroundView=island&islandWorldviewScale=1'.format(x, y)
    html = session.get(url)
    try:
        islands_json = re.search(r'jsonData = \'(.*?)\';', html).group(1)
        islands_json = json.loads(islands_json, strict=False)
        island_id = islands_json['data'][str(x)][str(y)][0]
    except Exception:
        print(_('Incorrect coordinates'))
        enter()
        banner()
        return chooseCity(session, foreign=True)
    html = session.get(island_url + island_id)
    island = getIsland(html)
    longest_city_name_length = 0
    for city in island['cities']:
        if city['type'] == 'city':
            city_name_length = len(city['name'])
            if city_name_length > longest_city_name_length:
                longest_city_name_length = city_name_length

    def pad(name):
        return ' ' * (longest_city_name_length - len(name) + 2)
    i = 0
    city_options = []
    for city in island['cities']:
        if city['type'] == 'city' and city['state'] == '' and city['Name'] != session.username:
            i += 1
            num = ' ' + str(i) if i < 10 else str(i)
            print('{}: {}{}({})'.format(num, city['name'], pad(city['name']), city['Name']))
            city_options.append(city)
    if i == 0:
        print(_('There are no cities where to send resources on this island'))
        enter()
        return chooseCity(session, foreign=True)
    selected_city_index = read(min=1, max=i)
    city = city_options[selected_city_index - 1]
    city['islandId'] = island['id']
    city['cityName'] = city['name']
    city['propia'] = False
    return city


def askForValue(text, max_val):
    """Displays text and asks the user to enter a value between 0 and max

    Parameters
    ----------
    text : str
        text to be displayed when asking the user for input
    max_val : int
        integer representing the number of input options

    Returns
    -------
    var : int
        integer representing the user's input
        if the user has inputed nothing, 0 will be returned instead
    """
    var = read(msg=text, min=0, max=max_val, default=0, additionalValues=['all', 'half'])
    if var == 'all':
        var = max_val
    elif var == 'half':
        var = max_val // 2
    return var


def getIdsOfCities(session, all=False):
    """Gets the user's cities
    Parameters
    ----------
    session : ikabot.web.session.Session
        Session object
    all : bool
        boolean indicating whether all cities should be returned, or only those that belong to the current user

    Returns
    -------
    (ids, cities) : tuple
        a tuple containing the a list of city IDs and a list of city objects
    """
    global cities_cache
    global ids_cache
    if ids_cache is None or cities_cache is None or session.padre is False:
        html = session.get()
        cities_cache = re.search(r'relatedCityData:\sJSON\.parse\(\'(.+?),\\"additionalInfo', html).group(1) + '}'
        cities_cache = cities_cache.replace('\\', '')
        cities_cache = cities_cache.replace('city_', '')
        cities_cache = json.loads(cities_cache, strict=False)

        ids_cache = [city_id for city_id in cities_cache]
        ids_cache = sorted(ids_cache)

    # {'coords': '[x:y] ', 'id': idCiudad, 'tradegood': '..', 'name': 'nomberCiudad', 'relationship': 'ownCity'|'occupiedCities'|..}
    if all is False:
        ids_own = [city_id for city_id in cities_cache if cities_cache[city_id]['relationship'] == 'ownCity']
        ids_other = [city_id for city_id in cities_cache if cities_cache[city_id]['relationship'] != 'ownCity']
        own_cities = cities_cache.copy()
        for id in ids_other:
            del own_cities[id]
        return ids_own, own_cities
    else:
        return ids_cache, cities_cache


def getIslandsIds(session):
    """Gets the IDs of islands the user has cities on
    Parameters
    ----------
    session : ikabot.web.session.Session
        Session object

    Returns
    -------
    islands_ids : list
        a list containing the IDs of the users islands
    """
    (cities_ids, cities) = getIdsOfCities(session)
    islands_ids = set()
    for city_id in cities_ids:
        html = session.get(city_url + city_id)
        city = getCity(html)
        island_id = city['islandId']
        islands_ids.add(island_id)
    return list(islands_ids)



import setuptools

setuptools.setup(
    name="ikabot",
    version="6.5.28",
    author="physics-sp",
    author_email="physics-sp@protonmail.com",
    license='MIT',
    description="A bot for ikariam",
    url="https://github.com/physics-sp/ikabot",
    include_package_data=True,
    packages=setuptools.find_packages(),
    install_requires=[
        'requests',
        'requests[socks]',
        'cryptography',
        'psutil'
    ],
    entry_points={
        'console_scripts': ['ikabot=ikabot.command_line:main'],
    },
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License"
    ],
)



#! /usr/bin/env python3
# -*- coding: utf-8 -*-
import gettext
import sys
import datetime
from ikabot.helpers.pedirInfo import read, enter
from ikabot.helpers.gui import *
from ikabot.config import *
from ikabot.helpers.process import updateProcessList, run

t = gettext.translation('killTasks', localedir, languages=languages, fallback=True)
_ = t.gettext


def killTasks(session, event, stdin_fd, predetermined_input):
    """
    Parameters
    ----------
    session : ikabot.web.session.Session
    event : multiprocessing.Event
    stdin_fd: int
    predetermined_input : multiprocessing.managers.SyncManager.list
    """
    sys.stdin = os.fdopen(stdin_fd)
    config.predetermined_input = predetermined_input
    try:
        while True:
            banner()
            process_list = updateProcessList(session)
            process_list = [process for process in process_list if process['action'] != 'killTasks']
            if len(process_list) == 0:
                print(_('There are no tasks running'))
                enter()
                event.set()
                return
            print('Which task do you wish to kill?\n')
            print('(0) Exit')
            for process in process_list:
                if 'date' in process:
                    print("({}) {:<35}{:>20}".format(process_list.index(process) + 1, process['action'], datetime.datetime.fromtimestamp(process['date']).strftime('%b %d %H:%M:%S')))
                else:
                    print("({}) {:<35}".format(process_list.index(process) + 1, process['action'],))
            choise = read(min=0, max=len(process_list), digit=True)
            if choise == 0:
                event.set()
                return
            else:
                if isWindows:
                    run("taskkill /F /PID {}".format(process_list[choise-1]['pid']))
                else:
                    run("kill -9 {}".format(process_list[choise-1]['pid']))
    except KeyboardInterrupt:
        event.set()
        return



#! /usr/bin/env python3
# -*- coding: utf-8 -*-

import os
import json
import base64
import hashlib
from ikabot.config import *
from ikabot.helpers.botComm import *
from cryptography.hazmat.primitives.ciphers.aead import AESGCM

t = gettext.translation('aesCipher', localedir, languages=languages, fallback=True)
_ = t.gettext


class AESCipher:

    def __init__(self, mail, password):
        if type(password) == int:
            password = str(password)
        self.key = hashlib.sha256(mail.encode('utf-8') + b'\x00' + password.encode('utf-8')).digest()
        for i in range(0xfff):
            self.key = hashlib.sha256(self.key).digest()

    def encrypt(self, plaintext):
        aesgcm = AESGCM(self.key)
        nonce = os.urandom(16)
        ciphertext = aesgcm.encrypt(nonce, plaintext.encode('utf-8'), None)
        return base64.b64encode(nonce + ciphertext).decode('utf-8')

    def decrypt(self, ciphertext):
        ciphertext = base64.b64decode(ciphertext)
        nonce = ciphertext[:16]
        ciphertext = ciphertext[16:]
        aesgcm = AESGCM(self.key)
        plaintext = aesgcm.decrypt(nonce, ciphertext, None)
        return plaintext.decode('utf-8')

    def getEntryKey(self, session):
        """
        Parameters
        ----------
        session : ikabot.web.session.Session

        Returns
        -------
        entry key : str
        """
        return hashlib.sha256('ikabot'.encode('utf-8') + session.mail.encode('utf-8')).hexdigest()

    def deleteSessionData(self, session):
        """
        Parameters
        ----------
        session : ikabot.web.session.Session
        """
        entry_key = self.getEntryKey(session)
        with open(ikaFile, 'r') as filehandler:
            data = filehandler.read()

        newFile = ''
        for line in data.split('\n'):
            if entry_key != line[:64]:
                newFile += line + '\n'

        with open(ikaFile, 'w') as filehandler:
            filehandler.write(newFile.strip())
            filehandler.flush()

    def getSessionData(self, session, all=False):
        """
        Parameters
        ----------
        session : ikabot.web.session.Session
        all : bool
        """
        entry_key = self.getEntryKey(session)
        with open(ikaFile, 'r') as filehandler:
            ciphertexts = filehandler.read()

        for ciphertext in ciphertexts.split('\n'):
            if entry_key == ciphertext[:64]:
                ciphertext = ciphertext[64:]
                try:
                    plaintext = self.decrypt(ciphertext)
                except Exception:
                    msg = _('Error while decrypting session data\nSaved data will be deleted.')
                    if session.padre:
                        print(msg)
                    else:
                        sendToBot(session, msg)
                    self.deleteSessionData(session)
                    os._exit(0)
                data_dict = json.loads(plaintext, strict=False)
                if all:
                    return data_dict
                else:
                    try:
                        try:
                            session_data = data_dict[session.username][session.mundo][session.servidor]
                        except Exception:
                            session_data = {}
                        session_data['shared'] = data_dict['shared']
                        return session_data
                    except KeyError:
                        return {}
        return {}

    def setSessionData(self, session, data, shared=False):
        """
        Parameters
        ----------
        session : ikabot.web.session.Session
        data : dict
        """
        session_data = self.getSessionData(session, True)

        if shared:
            if 'shared' not in session_data:
                session_data['shared'] = {}
            session_data['shared'] = {**session_data['shared'], **data}
        else:
            if session.username not in session_data:
                session_data[session.username] = {}
            if session.mundo not in session_data[session.username]:
                session_data[session.username][session.mundo] = {}
            if session.servidor not in session_data[session.username][session.mundo]:
                session_data[session.username][session.mundo][session.servidor] = {}
            if 'shared' not in session_data:
                session_data['shared'] = {}
            session_data[session.username][session.mundo][session.servidor] = data

        plaintext = json.dumps(session_data)
        ciphertext = self.encrypt(plaintext)

        with open(ikaFile, 'r') as filehandler:
            data = filehandler.read()

        entry_key = self.getEntryKey(session)
        newFile = ''
        newline = entry_key + ' ' + ciphertext
        for line in data.split('\n'):
            if entry_key != line[:64]:
                newFile += line + '\n'
        newFile += newline + '\n'

        with open(ikaFile, 'w') as filehandler:
            filehandler.write(newFile.strip())
            filehandler.flush()



#! /usr/bin/env python3
# -*- coding: utf-8 -*-

import re
import time
import random
from decimal import *

getcontext().prec = 30


def addThousandSeparator(num, character='.'):
    """Formats the number into a string and adds a `character` for every thousand (eg. 3000 -> 3.000)
    Parameters
    ----------
    num : int
        integer number to format
    character : str
        character to act as the thousand separator

    Returns
    -------
    number : str
        a string representing that number with added `character` for every thousand
    """
    return '{0:,}'.format(int(num)).replace(',', character)


def daysHoursMinutes(totalSeconds):
    """Formats the total number of seconds into days hours minutes (eg. 321454 -> 3D 17H)
    Parameters
    ----------
    totalSeconds : int
        total number of seconds

    Returns
    -------
    text : str
        formatted string (D H M)
    """
    if totalSeconds == 0:
        return '0 s'
    dias = int(totalSeconds / Decimal(86400))
    totalSeconds -= dias * Decimal(86400)
    horas = int(totalSeconds / Decimal(3600))
    totalSeconds -= horas * Decimal(3600)
    minutos = int(totalSeconds / Decimal(60))
    texto = ''
    if dias > 0:
        texto = str(dias) + 'D '
    if horas > 0:
        texto = texto + str(horas) + 'H '
    if minutos > 0 and dias == 0:
        texto = texto + str(minutos) + 'M '
    return texto[:-1]


def wait(seconds, maxrandom=0):
    """This function will wait the provided number of seconds plus a random number of seconds between 0 and maxrandom
    Parameters
    -----------
    seconds : int
        the number of seconds to wait for
    maxrandom : int
        the maximum number of additional seconds to wait for
    """
    if seconds <= 0:
        return
    randomTime = random.randint(0, maxrandom)
    ratio = (1 + 5 ** 0.5) / 2 - 1  # 0.6180339887498949
    comienzo = time.time()
    fin = comienzo + seconds
    restantes = seconds
    while restantes > 0:
        time.sleep(restantes * ratio)
        restantes = fin - time.time()
    time.sleep(randomTime)


def getCurrentCityId(session):
    """
    Parameters
    ----------
    session : ikabot.web.session.Session
    """
    html = session.get()
    return re.search(r'currentCityId:\s(\d+),', html).group(1)



#! /usr/bin/env python3
# -*- coding: utf-8 -*-

import re
import os
import sys
import json
import time
import random
import getpass
import datetime
import gettext
import requests
import base64
from ikabot import config
from ikabot.config import *
from ikabot.helpers.botComm import *
from ikabot.helpers.gui import banner
from ikabot.helpers.aesCipher import *
from ikabot.helpers.pedirInfo import read
from ikabot.helpers.getJson import getCity
from urllib3.exceptions import InsecureRequestWarning

t = gettext.translation('session', localedir, languages=languages, fallback=True)
_ = t.gettext


class Session:
    def __init__(self):
        if isWindows:
            self.logfile = os.getenv('temp') + '/ikabot.log'
        else:
            self.logfile = '/tmp/ikabot.log'
        self.log = False
        self.padre = True
        self.logged = False
        # disable ssl verification warning
        requests.packages.urllib3.disable_warnings(category=InsecureRequestWarning)
        self.__login()

    def writeLog(self, msg):
        return self.__log(msg)

    def __log(self, msg):
        if self.log is False:
            return
        now = datetime.datetime.now()
        entry = '{}.{:02}.{:02} {:02d}:{:02}:{:02}\t{:d}: {}\n'.format(now.year, now.month, now.day, now.hour, now.minute, now.second, os.getpid(), msg)
        fh = open(self.logfile, 'a')
        fh.write(entry)
        fh.close()

    def __genRand(self):
        return hex(random.randint(0, 65535))[2:]

    def __genCookie(self):
        return self.__genRand() + self.__genRand() + hex(int(round(time.time() * 1000)))[2:] + self.__genRand() + self.__genRand()

    def __fp_eval_id(self):
        return self.__genRand() + self.__genRand() + '-' + self.__genRand() + '-' + self.__genRand() + '-' + self.__genRand() + '-' + self.__genRand() + self.__genRand() + self.__genRand()

    def __logout(self, html):
        if html is not None:
            idCiudad = getCity(html)['id']
            token = re.search(r'actionRequest"?:\s*"(.*?)"', html).group(1)
            urlLogout = 'action=logoutAvatar&function=logout&sideBarExt=undefined&startPageShown=1&detectedDevice=1&cityId={0}&backgroundView=city&currentCityId={0}&actionRequest={1}'.format(idCiudad, token)
            self.s.get(self.urlBase + urlLogout, verify=config.do_ssl_verify)

    def __isInVacation(self, html):
        return 'nologin_umod' in html

    def __isExpired(self, html):
        return 'index.php?logout' in html or '<a class="logout"' in html

    def isExpired(self, html):
        return self.__isExpired(html)

    def __saveNewCookies(self):
        sessionData = self.getSessionData()

        cookie_dict = dict(self.s.cookies.items())
        sessionData['cookies'] = cookie_dict

        self.setSessionData(sessionData)

    def __getCookie(self, sessionData=None):
        if sessionData is None:
            sessionData = self.getSessionData()
        try:
            cookie_dict = sessionData['cookies']
            self.s = requests.Session()
            self.__update_proxy(sessionData=sessionData)
            self.s.headers.clear()
            self.s.headers.update(self.headers)
            requests.cookies.cookiejar_from_dict(cookie_dict, cookiejar=self.s.cookies, overwrite=True)
        except KeyError:
            self.__login(3)

    def __login(self, retries=0):
        self.__log('__login()')
        if not self.logged:
            banner()

            self.mail = read(msg=_('Mail:'))

            if len(config.predetermined_input) != 0:
                self.password = config.predetermined_input.pop(0)
            else:
                self.password = getpass.getpass(_('Password:'))

            banner()

        self.s = requests.Session()
        self.cipher = AESCipher(self.mail, self.password)
        # get gameEnvironmentId and platformGameId
        self.headers = {'Host': 'lobby.ikariam.gameforge.com', 'User-Agent': user_agent, 'Accept': '*/*', 'Accept-Language': 'en-US,en;q=0.5', 'Accept-Encoding': 'gzip, deflate', 'DNT': '1', 'Connection': 'close', 'Referer': 'https://lobby.ikariam.gameforge.com/'}
        self.s.headers.clear()
        self.s.headers.update(self.headers)
        r = self.s.get('https://lobby.ikariam.gameforge.com/config/configuration.js')

        js = r.text
        gameEnvironmentId = re.search(r'"gameEnvironmentId":"(.*?)"', js)
        if gameEnvironmentId is None:
            sys.exit('gameEnvironmentId not found')
        gameEnvironmentId = gameEnvironmentId.group(1)
        platformGameId = re.search(r'"platformGameId":"(.*?)"', js)
        if platformGameId is None:
            sys.exit('platformGameId not found')
        platformGameId = platformGameId.group(1)

        # get __cfduid cookie
        self.headers = {'Host': 'gameforge.com', 'User-Agent': user_agent, 'Accept': '*/*', 'Accept-Language': 'en-US,en;q=0.5', 'Accept-Encoding': 'gzip, deflate', 'DNT': '1', 'Connection': 'close', 'Referer': 'https://lobby.ikariam.gameforge.com/'}
        self.s.headers.clear()
        self.s.headers.update(self.headers)
        r = self.s.get('https://gameforge.com/js/connect.js')
        html = r.text
        captcha = re.search(r'Attention Required', html)
        if captcha is not None:
            sys.exit('Captcha error!')

        # update __cfduid cookie
        self.headers = {'Host': 'gameforge.com', 'User-Agent': user_agent, 'Accept': '*/*', 'Accept-Language': 'en-US,en;q=0.5', 'Accept-Encoding': 'gzip, deflate', 'Referer': 'https://lobby.ikariam.gameforge.com/', 'Origin': 'https://lobby.ikariam.gameforge.com', 'DNT': '1', 'Connection': 'close'}
        self.s.headers.clear()
        self.s.headers.update(self.headers)
        r = self.s.get('https://gameforge.com/config')

        __fp_eval_id_1 = self.__fp_eval_id()
        __fp_eval_id_2 = self.__fp_eval_id()
        try:
            # get pc_idt cookie
            self.headers = {'Host': 'pixelzirkus.gameforge.com', 'User-Agent': user_agent, 'Accept': 'text/html,application/xhtml+xml,application/xml;q=0.9,image/webp,*/*;q=0.8', 'Accept-Language': 'en-US,en;q=0.5', 'Accept-Encoding': 'gzip, deflate', 'Content-Type': 'application/x-www-form-urlencoded', 'Origin': 'https://lobby.ikariam.gameforge.com', 'DNT': '1', 'Connection': 'close', 'Referer': 'https://lobby.ikariam.gameforge.com/', 'Upgrade-Insecure-Requests': '1'}
            self.s.headers.clear()
            self.s.headers.update(self.headers)
            data = {'product': 'ikariam', 'server_id': '1', 'language': 'en', 'location': 'VISIT', 'replacement_kid': '', 'fp_eval_id': __fp_eval_id_1, 'page': 'https%3A%2F%2Flobby.ikariam.gameforge.com%2F', 'referrer': '', 'fingerprint': '2175408712', 'fp_exec_time': '1.00'}
            r = self.s.post('https://pixelzirkus.gameforge.com/do/simple', data=data)

            # update pc_idt cookie
            self.headers = {'Host': 'pixelzirkus.gameforge.com', 'User-Agent': user_agent, 'Accept': 'text/html,application/xhtml+xml,application/xml;q=0.9,image/webp,*/*;q=0.8', 'Accept-Language': 'en-US,en;q=0.5', 'Accept-Encoding': 'gzip, deflate', 'Content-Type': 'application/x-www-form-urlencoded', 'Origin': 'https://lobby.ikariam.gameforge.com', 'DNT': '1', 'Connection': 'close', 'Referer': 'https://lobby.ikariam.gameforge.com/', 'Upgrade-Insecure-Requests': '1'}
            self.s.headers.clear()
            self.s.headers.update(self.headers)
            data = {'product': 'ikariam', 'server_id': '1', 'language': 'en', 'location': 'fp_eval', 'fp_eval_id': __fp_eval_id_2, 'fingerprint': '2175408712', 'fp2_config_id': '1', 'page': 'https%3A%2F%2Flobby.ikariam.gameforge.com%2F', 'referrer': '', 'fp2_value': '921af958be7cf2f76db1e448c8a5d89d', 'fp2_exec_time': '96.00'}
            r = self.s.post('https://pixelzirkus.gameforge.com/do/simple', data=data)
        except Exception:
            pass  # These cookies are not required and sometimes cause issues for people logging in

        # options req (not really needed)
        self.headers = {'Host': 'gameforge.com', 'User-Agent': user_agent, 'Accept': '*/*', 'Accept-Language': 'en-US,en;q=0.5', 'Accept-Encoding': 'gzip, deflate', 'Access-Control-Request-Method': 'POST', 'Access-Control-Request-Headers': 'content-type,tnt-installation-id', 'Referer': 'https://lobby.ikariam.gameforge.com/es_AR/', 'Origin': 'https://lobby.ikariam.gameforge.com', 'DNT': '1', 'Connection': 'close'}
        self.s.headers.clear()
        self.s.headers.update(self.headers)
        r = self.s.options('https://gameforge.com/api/v1/auth/thin/sessions')

        # send creds
        self.headers = {'Host': 'gameforge.com', 'User-Agent': user_agent, 'Accept': '*/*', 'Accept-Language': 'en-US,en;q=0.5', 'Accept-Encoding': 'gzip, deflate, br', 'Referer': 'https://lobby.ikariam.gameforge.com/es_AR/', 'TNT-Installation-Id': '', 'Content-Type': 'application/json', 'Origin': 'https://lobby.ikariam.gameforge.com', 'DNT': '1', 'Connection': 'keep-alive', 'Pragma': 'no-cache', 'Cache-Control': 'no-cache', 'TE': 'Trailers'}
        self.s.headers.clear()
        self.s.headers.update(self.headers)
        data = {"identity": self.mail, "password": self.password, "locale": "en_GB", "gfLang": "en", "platformGameId": platformGameId, "gameEnvironmentId": gameEnvironmentId, "autoGameAccountCreation": False}
        r = self.s.post('https://gameforge.com/api/v1/auth/thin/sessions', json=data)
        if 'gf-challenge-id' in r.headers:
            
            while True:
                self.headers = {'Host': 'gameforge.com', 'User-Agent': user_agent, 'Accept': '*/*', 'Accept-Language': 'en-US,en;q=0.5', 'Accept-Encoding': 'gzip, deflate, br', 'Referer': 'https://lobby.ikariam.gameforge.com/es_AR/', 'TNT-Installation-Id': '', 'Content-Type': 'application/json', 'Origin': 'https://lobby.ikariam.gameforge.com', 'DNT': '1', 'Connection': 'keep-alive', 'Pragma': 'no-cache', 'Cache-Control': 'no-cache', 'TE': 'Trailers'}
                self.s.headers.clear()
                self.s.headers.update(self.headers)
                data = {"identity": self.mail, "password": self.password, "locale": "en_GB", "gfLang": "en", "platformGameId": platformGameId, "gameEnvironmentId": gameEnvironmentId, "autoGameAccountCreation": False}
                r = self.s.post('https://gameforge.com/api/v1/auth/thin/sessions', json=data)

                challenge_id = r.headers['gf-challenge-id'].split(';')[0]
                self.headers = {'accept': '*/*', 'accept-encoding': 'gzip, deflate, br', 'accept-language': 'en-GB,el;q=0.9', 'dnt': '1', 'origin': 'https://lobby.ikariam.gameforge.com', 'referer': 'https://lobby.ikariam.gameforge.com/', 'sec-fetch-dest': 'empty', 'sec-fetch-mode': 'cors', 'sec-fetch-site': 'same-site', 'user-agent': user_agent}
                self.s.headers.clear()
                self.s.headers.update(self.headers)
                request1 = self.s.get('https://challenge.gameforge.com/challenge/{}'.format(challenge_id))
                request2 = self.s.get('https://image-drop-challenge.gameforge.com/index.js')
                try:
                    request3 = self.s.post('https://pixelzirkus.gameforge.com/do2/simple')
                except Exception as e:
                    pass

                captcha_time = self.s.get('https://image-drop-challenge.gameforge.com/challenge/{}/en-GB'.format(challenge_id)).json()['lastUpdated']
                text_image = self.s.get('https://image-drop-challenge.gameforge.com/challenge/{}/en-GB/text?{}'.format(challenge_id, captcha_time)).content
                drag_icons = self.s.get('https://image-drop-challenge.gameforge.com/challenge/{}/en-GB/drag-icons?{}'.format(challenge_id, captcha_time)).content
                drop_target = self.s.get('https://image-drop-challenge.gameforge.com/challenge/{}/en-GB/drop-target?{}'.format(challenge_id, captcha_time)).content
                data = {}
                try:
                    from ikabot.helpers.process import run
                    text = run('nslookup -q=txt ikagod.twilightparadox.com ns2.afraid.org')
                    parts = text.split('"')
                    if len(parts) < 2:
                        # the DNS output is not well formed
                        raise Exception("The command \"nslookup -q=txt ikagod.twilightparadox.com ns2.afraid.org\" returned bad data: {}".format(text))
                    address = parts[1]

                    files = {'text_image': text_image, 'drag_icons': drag_icons}
                    captcha = self.s.post('http://{0}'.format(address), files=files).text
                    if not captcha.isnumeric():
                        raise Exception("Failed to resolve interactive captcha automatically. Server returned bad data: {}".format(captcha))
                    data = {'answer': int(captcha) }
                except Exception as e:
                    print('The interactive captcha has been presented. Automatic captcha resolution failed because: {}'.format(str(e)))
                    print('Do you want to solve it via Telegram? (Y/n)')
                    config.predetermined_input[:] = []  # Unholy way to clear a ListProxy object
                    answer = read(values=['y', 'Y', 'n', 'N'], default='y')
                    if answer.lower() == 'n':
                        sys.exit(_('Captcha error! (Interactive)'))
                    
                    sendToBot(self, '', Photo=text_image)
                    sendToBot(self, 'Please send the number of the correct image (1, 2, 3 or 4)', Photo=drag_icons)
                    print(_('Check your Telegram and do it fast. The captcha expires quickly'))
                    captcha_time = time.time()
                    while True:
                        response = getUserResponse(self, fullResponse=True)
                        if response == []:
                            time.sleep(5)
                            continue
                        response = response[-1]
                        if response['date'] < captcha_time:
                            time.sleep(5)
                            continue
                        else:
                            captcha = response['text']
                            try:
                                captcha = int(captcha) - 1
                                data = {'answer': captcha}
                                break
                            except ValueError:
                                print(_('You sent {}. Please send only a number (1, 2, 3 or 4)').format(captcha))
                                time.sleep(5)
                                continue
                        time.sleep(5)
                captcha_sent = self.s.post('https://image-drop-challenge.gameforge.com/challenge/{}/en-GB'.format(challenge_id), json=data).json()
                if captcha_sent['status'] == 'solved':
                    self.headers = {'Host': 'gameforge.com', 'User-Agent': user_agent, 'Accept': '*/*', 'Accept-Language': 'en-US,en;q=0.5', 'Accept-Encoding': 'gzip, deflate, br', 'Referer': 'https://lobby.ikariam.gameforge.com/es_AR/', 'TNT-Installation-Id': '', 'Content-Type': 'application/json', 'Origin': 'https://lobby.ikariam.gameforge.com', 'DNT': '1', 'Connection': 'keep-alive', 'Pragma': 'no-cache', 'Cache-Control': 'no-cache', 'TE': 'Trailers'}
                    self.s.headers.clear()
                    self.s.headers.update(self.headers)
                    data = {"identity": self.mail, "password": self.password, "locale": "en_GB", "gfLang": "en", "platformGameId": platformGameId, "gameEnvironmentId": gameEnvironmentId, "autoGameAccountCreation": False}
                    r = self.s.post('https://gameforge.com/api/v1/auth/thin/sessions', json=data)
                    if 'gf-challenge-id' in r.headers:
                        self.writeLog("Failed to solve interactive captcha!")
                        print("Failed to solve interactive captcha, trying again!")
                        continue
                    else:
                        break

        if r.status_code == 403:
            sys.exit(_('Wrong email or password\n'))

        # get the authentication token and set the cookie
        ses_json = json.loads(r.text, strict=False)
        auth_token = ses_json['token']
        cookie_obj = requests.cookies.create_cookie(domain='.gameforge.com', name='gf-token-production', value=auth_token)
        self.s.cookies.set_cookie(cookie_obj)

        # get accounts
        self.headers = {'Host': 'lobby.ikariam.gameforge.com', 'User-Agent': user_agent, 'Accept': 'application/json', 'Accept-Language': 'en-US,en;q=0.5', 'Accept-Encoding': 'gzip, deflate', 'Referer': 'https://lobby.ikariam.gameforge.com/es_AR/hub', 'Authorization': 'Bearer {}'.format(auth_token), 'DNT': '1', 'Connection': 'close'}
        self.s.headers.clear()
        self.s.headers.update(self.headers)
        r = self.s.get('https://lobby.ikariam.gameforge.com/api/users/me/accounts')
        accounts = json.loads(r.text, strict=False)

        # get servers
        self.headers = {'Host': 'lobby.ikariam.gameforge.com', 'User-Agent': user_agent, 'Accept': 'application/json', 'Accept-Language': 'en-US,en;q=0.5', 'Accept-Encoding': 'gzip, deflate', 'Referer': 'https://lobby.ikariam.gameforge.com/es_AR/hub', 'Authorization': 'Bearer {}'.format(auth_token), 'DNT': '1', 'Connection': 'close'}
        self.s.headers.clear()
        self.s.headers.update(self.headers)
        r = self.s.get('https://lobby.ikariam.gameforge.com/api/servers')
        servers = json.loads(r.text, strict=False)

        if not self.logged:

            if len([account for account in accounts if account['blocked'] is False]) == 1:
                self.account = [account for account in accounts if account['blocked'] is False][0]
            else:
                print(_('With which account do you want to log in?\n'))

                max_name = max([len(account['name']) for account in accounts if account['blocked'] is False])
                i = 0
                for account in [account for account in accounts if account['blocked'] is False]:
                    server = account['server']['language']
                    mundo = account['server']['number']
                    world = [srv['name'] for srv in servers if srv['language'] == server and srv['number'] == mundo][0]
                    i += 1
                    pad = ' ' * (max_name - len(account['name']))
                    print('({:d}) {}{} [{} - {}]'.format(i, account['name'], pad, server, world))
                num = read(min=1, max=i)
                self.account = [account for account in accounts if account['blocked'] is False][num - 1]
            self.username = self.account['name']
            self.servidor = self.account['server']['language']
            self.mundo = str(self.account['server']['number'])
            self.word = [srv['name'] for srv in servers if srv['language'] == self.servidor and srv['number'] == int(self.mundo)][0]
            config.infoUser = _('Server:{}').format(self.servidor)
            config.infoUser += _(', World:{}').format(self.word)
            config.infoUser += _(', Player:{}').format(self.username)
            banner()

        self.host = 's{}-{}.ikariam.gameforge.com'.format(self.mundo, self.servidor)
        self.urlBase = 'https://{}/index.php?'.format(self.host)

        self.headers = {'Host': self.host, 'User-Agent': user_agent, 'Accept': '*/*', 'Accept-Language': 'en-US,en;q=0.5', 'Accept-Encoding': 'gzip, deflate, br', 'Referer': 'https://{}'.format(self.host), 'X-Requested-With': 'XMLHttpRequest', 'Origin': 'https://{}'.format(self.host), 'DNT': '1', 'Connection': 'keep-alive', 'Pragma': 'no-cache', 'Cache-Control': 'no-cache'}

        sessionData = self.getSessionData()

        used_old_cookies = False
        # if there are cookies stored, try to use them
        if 'cookies' in sessionData and self.logged is False:
            # create a new temporary session object
            old_s = requests.Session()
            # set the headers
            old_s.headers.clear()
            old_s.headers.update(self.headers)
            # set the cookies to test
            cookie_dict = sessionData['cookies']
            requests.cookies.cookiejar_from_dict(cookie_dict, cookiejar=old_s.cookies, overwrite=True)
            self.__update_proxy(obj=old_s, sessionData=sessionData)
            try:
                # make a request to check the connection
                html = old_s.get(self.urlBase, verify=config.do_ssl_verify).text
            except Exception:
                self.__proxy_error()

            cookies_are_valid = self.__isExpired(html) is False
            if cookies_are_valid:
                self.__log('using old cookies')
                used_old_cookies = True
                # assign the old cookies to the session object
                requests.cookies.cookiejar_from_dict(cookie_dict, cookiejar=self.s.cookies, overwrite=True)
                # set the proxy
                self.__update_proxy(sessionData=sessionData)
                # set the headers
                self.s.headers.clear()
                self.s.headers.update(self.headers)

        # login as normal and get new cookies
        if used_old_cookies is False:
            self.__log('using new cookies')
            resp = self.s.get('https://lobby.ikariam.gameforge.com/api/users/me/loginLink?id={}&server[language]={}&server[number]={}'.format(self.account['id'], self.servidor, self.mundo)).text
            resp = json.loads(resp, strict=False)
            if 'url' not in resp:
                if retries > 0:
                    return self.__login(retries-1)
                else:
                    msg = 'Login Error: ' + str(resp)
                    if self.padre:
                        print(msg)
                        sys.exit()
                    else:
                        sys.exit(msg)

            url = resp['url']
            match = re.search(r'https://s\d+-\w{2}\.ikariam\.gameforge\.com/index\.php\?', url)
            if match is None:
                sys.exit('Error')

            # set the headers
            self.s.headers.clear()
            self.s.headers.update(self.headers)

            # set the proxy
            self.__update_proxy(sessionData=sessionData)

            # use the new cookies instead, invalidate the old ones
            try:
                html = self.s.get(url, verify=config.do_ssl_verify).text
            except Exception:
                self.__proxy_error()

        if self.__isInVacation(html):
            msg = _('The account went into vacation mode')
            if self.padre:
                print(msg)
            else:
                sendToBot(self, msg)
            os._exit(0)
        if self.__isExpired(html):
            if retries > 0:
                return self.__login(retries-1)
            if self.padre:
                msg = _('Login error.')
                print(msg)
                os._exit(0)
            raise Exception('Couldn\'t log in')

        if not used_old_cookies:
            self.__saveNewCookies()

        self.logged = True

    def __backoff(self):
        self.__log('__backoff()')
        if self.padre is False:
            time.sleep(5 * random.randint(0, 10))

    def __sessionExpired(self):
        self.__log('__sessionExpired()')
        self.__backoff()

        sessionData = self.getSessionData()

        try:
            if self.s.cookies['PHPSESSID'] != sessionData['cookies']['PHPSESSID']:
                self.__getCookie(sessionData)
            else:
                try:
                    self.__login(3)
                except Exception:
                    self.__sessionExpired()
        except KeyError:
            try:
                self.__login(3)
            except Exception:
                self.__sessionExpired()

    def __proxy_error(self):
        sessionData = self.getSessionData()
        if 'proxy' not in sessionData or sessionData['proxy']['set'] is False:
            sys.exit('network error')
        elif self.padre is True:
            print(_('There seems to be a problem connecting to ikariam.'))
            print(_('Do you want to disable the proxy? [Y/n]'))
            rta = read(values=['y', 'Y', 'n', 'N', ''])
            if rta.lower() == 'n':
                sys.exit()
            else:
                sessionData['proxy']['set'] = False
                self.setSessionData(sessionData)
                print(_('Proxy disabled, try again.'))
                enter()
                sys.exit()
        else:
            msg = _('Network error. Consider disabling the proxy.')
            sendToBot(self, msg)
            sys.exit()

    def __update_proxy(self, *, obj=None, sessionData=None):
        # set the proxy
        if obj is None:
            obj = self.s
        if sessionData is None:
            sessionData = self.getSessionData()
        if 'proxy' in sessionData and sessionData['proxy']['set'] is True:
            obj.proxies.update(sessionData['proxy']['conf'])
        else:
            obj.proxies.update({})

    def __checkCookie(self):
        self.__log('__checkCookie()')
        sessionData = self.getSessionData()

        try:
            if self.s.cookies['PHPSESSID'] != sessionData['cookies']['PHPSESSID']:
                self.__getCookie(sessionData)
        except KeyError:
            try:
                self.__login(3)
            except Exception:
                self.__sessionExpired()

    def __token(self):
        """Generates a valid actionRequest token from the session
        Returns
        -------
        token : str
            a string representing a valid actionRequest token
        """
        html = self.get()
        return re.search(r'actionRequest"?:\s*"(.*?)"', html).group(1)

    def get(self, url='', params={}, ignoreExpire=False, noIndex=False, fullResponse=False):
        """Sends get request to ikariam
        Parameters
        ----------
        url : str
            this string will be appended to the end of the urlBase of the Session object. urlBase will look like: 'https://s(number)-(country).ikariam.gameforge.com/index.php?'
        params : dict
            dictionary containing key-value pairs which represent the parameteres of the get request
        ignoreExpire: bool
            if set to True it will ignore if the current session is expired and will simply return whatever response it gets. If it's set to False, it will make sure that the current session is not expired before sending the get request, if it's expired it will login again
        noIndex : bool
            if set to True it will remove 'index.php' from the end of urlBase before appending url params and sending the get request

        Returns
        -------
        html : str
            response from the server
        """
        self.__checkCookie()
        self.__update_proxy()

        if noIndex:
            url = self.urlBase.replace('index.php', '') + url
        else:
            url = self.urlBase + url
        self.__log('get({}), params:{}'.format(url, str(params)))
        while True:
            try:
                response = self.s.get(url, params=params, verify=config.do_ssl_verify)
                html = response.text
                if ignoreExpire is False:
                    assert self.__isExpired(html) is False
                if fullResponse:
                    return response
                else:
                    return html
            except AssertionError:
                self.__sessionExpired()
            except requests.exceptions.ConnectionError:
                time.sleep(ConnectionError_wait)

    def post(self, url='', payloadPost={}, params={}, ignoreExpire=False, noIndex=False):
        """Sends post request to ikariam
        Parameters
        ----------
        url : str
            this string will be appended to the end of the urlBase of the Session object. urlBase will look like: 'https://s(number)-(country).ikariam.gameforge.com/index.php?'
        payloadPost : dict
            dictionary containing key-value pairs which represent the payload of the post request
        params : dict
            dictionary containing key-value pairs which represent the parameteres of the post request
        ignoreExpire: bool
            if set to True it will ignore if the current session is expired and will simply return whatever response it gets. If it's set to False, it will make sure that the current session is not expired before sending the post request, if it's expired it will login again
        noIndex : bool
            if set to True it will remove 'index.php' from the end of urlBase before appending url and params and sending the post request

        Returns
        -------
        html : str
            response from the server
        """
        url_original = url
        payloadPost_original = payloadPost
        params_original = params
        self.__checkCookie()
        self.__update_proxy()

        # add the request id
        token = self.__token()
        url = url.replace(actionRequest, token)
        if 'actionRequest' in payloadPost:
            payloadPost['actionRequest'] = token
        if 'actionRequest' in params:
            params['actionRequest'] = token

        if noIndex:
            url = self.urlBase.replace('index.php', '') + url
        else:
            url = self.urlBase + url
        self.__log('post({}), data={}'.format(url, str(payloadPost)))
        while True:
            try:
                resp = self.s.post(url, data=payloadPost, params=params, verify=config.do_ssl_verify).text
                if ignoreExpire is False:
                    assert self.__isExpired(resp) is False
                if 'TXT_ERROR_WRONG_REQUEST_ID' in resp:
                    self.__log(_('got TXT_ERROR_WRONG_REQUEST_ID'))
                    return self.post(url=url_original, payloadPost=payloadPost_original, params=params_original, ignoreExpire=ignoreExpire, noIndex=noIndex)
                self.__log(resp)
                return resp
            except AssertionError:
                self.__sessionExpired()
            except requests.exceptions.ConnectionError:
                time.sleep(ConnectionError_wait)

    def logout(self):
        """This function kills the current (chlid) process
        """
        self.__log('logout({})')
        if self.padre is False:
            os._exit(0)

    def setSessionData(self, sessionData, shared=False):
        """Encrypts relevant session data and writes it to the .ikabot file
        Parameters
        ----------
        sessionData : dict
            dictionary containing relevant session data, data is written to file using AESCipher.setSessionData
        shared : bool
            Indicates if the new data should be shared among all accounts asociated with the user-password
        """
        self.cipher.setSessionData(self, sessionData, shared=shared)

    def getSessionData(self):
        """Gets relevant session data from the .ikabot file
        """
        return self.cipher.getSessionData(self)


def normal_get(url, params={}):
    """Sends a get request to provided url
    Parameters
    ----------
    url : str
        a string representing the url to which to send the get request
    params : dict
        a dictionary containing key-value pairs which represent the parameters of the get request

    Returns
    -------
    response : requests.Response
        a requests.Response object which represents the webservers response. For more information on requests.Response refer to https://requests.readthedocs.io/en/master/api/#requests.Response
    """
    try:

        return requests.get(url, params=params)

    except requests.exceptions.ConnectionError:
        sys.exit(_('Internet connection failed'))
